/*=================
Arduino code for a Smart Environments project
Sends & senses data about potential mosquito hazard
via bluetooth to an app, can take action automatically
or be manually overridden to manipulate window screens

Code by Nina Vroom, uses SoftwareSerial library for
bluetooth and DHT library for sensors
=================*/

#include <SoftwareSerial.h>
#include "DHT.h"
#define DHTPIN 2  // indoor/outdoor sensors
#define DHTPININ 4
#define DHTTYPE DHT11

#define CWPIN 3 // motor pins
#define CCWPIN 5

DHT dht(DHTPIN, DHTTYPE);
DHT dhtIn(DHTPININ, DHTTYPE);
SoftwareSerial EEBlue(10, 11); // RX | TX

unsigned long time;                   // keeps track of time for polling rate
float avgTemp;
float avgRHumidity;
float avgTempIn;
float avgRHumidityIn;
int count;

char incomingValue = 0;
String state;
boolean manualOverride = false;

void setup() {
  Serial.begin(9600);                 // open serial
  EEBlue.begin(9600);
  state = "safe";
  pinMode(CWPIN, OUTPUT);
  pinMode(CCWPIN, OUTPUT);
  dht.begin(); // initialize the sensors
  dhtIn.begin();
}

void loop() {
  if (millis()>time+2149){             // poll every ~2 seconds
    avgTemp = 0;
    avgRHumidity = 0;
    avgTempIn = 0;
    avgRHumidityIn = 0;
    count = 0;
    while(millis()<=time+2200) {      // continously record for 50ms
      count ++;
      avgTemp += dht.readTemperature();
      avgRHumidity += dht.readHumidity();
      avgTempIn += dhtIn.readTemperature();
      avgRHumidityIn += dhtIn.readHumidity();
    }
    avgTemp = avgTemp/count;          // average values from 50ms
    avgRHumidity = avgRHumidity/count;
    avgTempIn = avgTempIn/count;
    avgRHumidityIn = avgRHumidityIn/count;
    
    time=millis();
    
    if (possibleMozzies(avgTemp, avgRHumidity) && state == "safe") {                  // trigger 'danger' action, such as running a motor for a time interval
      state = "danger";
      if(!manualOverride) {
        Serial.println("Mosquito danger, rolling down window screens...");
        turnCw();
      } else {
        // TODO: broadcast to phone current status
      }
    } else if (!possibleMozzies(avgTemp, avgRHumidity) && state == "danger") {         // trigger 'safe' action
      state = "safe";
      if(!manualOverride) {
        Serial.println("No more danger, rolling up window screens...");
        turnCcw();
      } else {
        // TODO: broadcast to phone current status
      }
    }

    if (EEBlue.available() > 0) {           // read bluetooth for override
      incomingValue = EEBlue.read();
      if(incomingValue == 'm') {
        manualOverride = !manualOverride;
      }
      if (true) {
        if(incomingValue == '1') {
          turnCw();
          Serial.println("spincw");
        } else if (incomingValue == '0') {
          turnCcw();
          Serial.println("spinccw");
        }
      }
    }

    EEBlue.println(String(avgTemp,2)+"|"+String(avgRHumidity,2)+"|"+String(avgTempIn,2)+"|"+String(avgRHumidityIn,2)); // send data over bluetooth
  }
}

bool possibleMozzies(float temp, float rHumidity) {                                   // check for possible mosquitoes
  if(temp >= 22 && temp <= 32 && rHumidity >= 60 && rHumidity <= 90) {
    return true;
  } else if(temp <= 19 || temp >= 35 || rHumidity <= 50 || rHumidity >= 95) {
    return false;
  }
  
  if (state == "danger") {
    return true;
  } else {
    return false;
  }
}

//================ 
// Motor functions
//================
void turnCw() {
  turnStop();
  analogWrite(CWPIN, 180);
  analogWrite(CCWPIN, 0);
  delay(300);
  turnStop();
}

void turnCcw() {
  turnStop();
  analogWrite(CWPIN, 0);
  analogWrite(CCWPIN, 180);
  delay(300);
  turnStop();
}

void turnStop() {
  analogWrite(CWPIN, 0);
  analogWrite(CCWPIN, 0);
  delay(50);
}
